library(RColorBrewer)
library(xtable)

load("DEsims.sdoffset0.0.RData")
sdoffset <- 0.0                            ## SD offset library size

p <- 20000                                 ## total number of genes
pDE <- 100                                 ## number of DE genes
fc <- 1.5                                  ## fold-change
n <- 70                                    ## sample size
grp <- rep(1:2, each=n/2)                  ## sample group indicator variable
nsims <- 100                               ## number of simulations
qval.cutoffs <- seq(0, 0.1, by=0.001)      ## q-value cutoffs

pp <- colorRampPalette(brewer.pal(12, "Paired"))(12)
collist <- c(pp[6], pp[2], pp[1], pp[10], pp[9], pp[5], pp[7], pp[8], pp[12])

all.eFDR.sims <- vector(mode="list", length=length(eFDRlist[[1]]))
names(all.eFDR.sims) <- names(eFDRlist[[1]])
all.eFDR.sims <- lapply(all.eFDR.sims, function(x) matrix(NA, nrow=length(qval.cutoffs), ncol=0))
all.mse.sims <- vector(mode="list", length=length(eFDRlist[[1]]))
names(all.mse.sims) <- names(eFDRlist[[1]])
all.mse.sims <- lapply(all.mse.sims, function(x) matrix(NA, nrow=3, ncol=3, dimnames=list(c("100", "1000", "2000"), c("1.5", "2", "4"))))

pdf(file=paste0("qvalXeFDR.", sprintf("sdoff%.1f.pdf", sdoffset)), height=7, width=7, useDingbats=FALSE)
metnames1 <- c("tweeDEseq", "DESeq - PgOn", "DESeq - Pmax", "DESeq - CgOn", "DESeq - Cmax")
metnames2 <- c("edgeR - (def)", "edgeR - QLF (def)", "edgeR - Df1", "edgeR - QLF Df1")
par(mfrow=c(3, 3), mar=c(4, 4, 1, 1))
for (pDE in c(100, 1000, 2000)) {
  for (fc in c(1.5, 2, 4)) {
    sim.name <- sprintf("qvalXeFDR.p%d.pDE%d.n%d.fc%.1f.sdoff%.1f", p, pDE, n, fc, sdoffset)
    all.eFDR <- eFDRlist[[sim.name]]
    for (met in names(all.eFDR)) {
      all.eFDR.sims[[met]] <- cbind(all.eFDR.sims[[met]], all.eFDR[[met]])
      all.mse.sims[[met]][as.character(pDE), as.character(fc)] <- sum((qval.cutoffs-rowMeans(all.eFDR[[met]]))^2)
    }

    par(mar=c(4, 4, 1, 1))
    if (pDE == 100 || fc == 1.5) {
      if (pDE == 100)
        par(mar=c(4, 4, 3, 1))
      if (fc == 1.5)
        par(mar=c(4, 5, 1, 1))
      if (pDE == 100 && fc == 1.5)
        par(mar=c(4, 5, 3, 1))
    }

    plot(qval.cutoffs, rowMeans(all.eFDR$tweeDEseq), type="l", lwd=3,
         col=collist[1], xlab="q-values", ylab="eFDR", ## ylim=c(0, 1)) ## , xlim=c(0, 1))
         ylim=range(sapply(all.eFDR, rowMeans), na.rm=TRUE), las=1)
    for (i in 2:length(all.eFDR)) {
      lines(qval.cutoffs, rowMeans(all.eFDR[[i]]), type="l", lwd=3, col=collist[i])
    }
    lines(qval.cutoffs, rowMeans(all.eFDR[[1]]), type="l", lwd=3, col=collist[1])
    if (pDE == 100)
      mtext(sprintf("Fold-change %.1f", fc), side=3, line=1)
    if (fc == 1.5)
      mtext(sprintf("#DE=%d", pDE), side=2, line=3.9)
    abline(0, 1, lwd=2, lty=2)
    if (pDE == 1000 && fc == 4)
      legend("topleft", metnames1, col=collist[1:5], lwd=3, lty=1, inset=0.01, cex=0.8)
    if (pDE == 2000 && fc == 4)
      legend("topleft", metnames2, col=collist[6:9], lwd=3, lty=1, inset=0.01, cex=0.8)
  }
}
dev.off()

metnames <- c(metnames1, metnames2)
names(metnames) <- names(all.eFDR)
all.mse.df <- data.frame(pDE=rep(c(100, 1000, 2000), each=9),
                         rank=rep(1:9, 3))
rnkmet <- data.frame(method=names(all.mse.sims), stringsAsFactors=FALSE)

for (pDE in c(100, 1000, 2000)) {
  for (fc in c(1.5, 2, 4)) {
    mse <- sapply(all.mse.sims, function(x, pDE, fc) x[pDE, fc], as.character(pDE), as.character(fc)) 
    fcstr <- sprintf("fc%.1f", fc)
    pDEstr <- sprintf("pDE%d", pDE)
    rnkmet[[paste0(fcstr, pDEstr)]][names(mse)] <- rank(mse)
    all.mse.df[[sprintf("fc%.1f", fc)]][which(all.mse.df$pDE == pDE)] <- sort(mse)
    all.mse.df[[sprintf("metFC%.1f", fc)]][which(all.mse.df$pDE == pDE)] <- metnames[names(sort(mse))]
  }
}
rnkmet[["method"]] <- metnames[rnkmet[["method"]]]

xtab <- xtable(all.mse.df, caption="xxx", digits=3)
print(xtab, file=sprintf("qvalXeFDRmse.sdoff%.1f.tex", sdoffset),
     include.rownames=FALSE)

xtab <- xtable(rnkmet, caption="xxx", digits=0)
print(xtab, file=sprintf("qvalXeFDRmse.sdoff%.1f.rnk.tex", sdoffset),
      include.rownames=FALSE)

all.mse.sims0.0 <- all.mse.sims
rnkmet0.0 <- rnkmet

save(all.mse.sims0.0, rnkmet0.0, file="msernkmet.sdoff0.0.RData")

all.pi0.r <- matrix(NA_real_, nrow=0, ncol=length(pi0list[[1]]))
colnames(all.pi0.r) <- names(pi0list[[1]])

pdf(file=paste0("estNrDEgenes.", sprintf("sdoff%.1f.pdf", sdoffset)), height=7, width=7, useDingbats=FALSE)
par(mar=c(4, 5, 1, 1))
par(mfrow=c(3, 3), mar=c(4, 4, 1, 1))
for (pDE in c(100, 1000, 2000)) {
  for (fc in c(1.5, 2, 4)) {
    sim.name <- sprintf("qvalXeFDR.p%d.pDE%d.n%d.fc%.1f.sdoff%.1f", p, pDE, n, fc, sdoffset)
    all.pi0 <- pi0list[[sim.name]]
    par(mar=c(4, 5, 1, 1))
    if (pDE == 100 || fc == 1.5) {
      if (pDE == 100)
        par(mar=c(4, 5, 3, 1))
      if (fc == 1.5)
        par(mar=c(4, 6, 1, 1))
      if (pDE == 100 && fc == 1.5)
        par(mar=c(4, 6, 3, 1))
    }

    boxplot(lapply(all.pi0, function(x, p) p*(1-x), p), names=1:length(all.pi0), ## rep("", 9),
            xlab="Method", ylab="Est. #DE genes", col=collist, las=1, cex.lab=1.2)
    abline(h=pDE, lwd=2, lty=2)

    if (pDE == 100)
      mtext(sprintf("Fold-change %.1f", fc), side=3, line=1)
    if (fc == 1.5)
      mtext(sprintf("#DE=%d", pDE), side=2, line=4.5)

    all.pi0.r <- rbind(all.pi0.r,
                       do.call(cbind, lapply(all.pi0, function(x, r) (1-x)/r, pDE/p)))
  }
}
dev.off()

all.pi0.r.sdoff0.0 <- all.pi0.r
save(all.pi0.r.sdoff0.0, file="all.pi0.r.sdoff0.0.RData")
