p <- 20000                                 ## total number of genes
pDE <- 100                                 ## number of DE genes
fc <- 1.5                                  ## fold-change
n <- 70                                    ## sample size
grp <- rep(1:2, each=n/2)                  ## sample group indicator variable
nsims <- 100                               ## number of simulations
sdoffset <- 0.5                            ## SD offset library size

library(qvalue)

eFDR <- function(pval, qval, de.genes, qval.cutoffs) {

  y <- sapply(qval.cutoffs, function(q, pval, qval, de.genes) {
                efdr <- 0
                ## de.calls <- names(pval)[pval <= pval[max(c(0,which(qval <= q)))]]
                if (any(qval <= q)) {
                  de.calls <- which(pval <= max(pval[which(qval <= q)]))
                  if (length(de.calls) > 0)
                    efdr <- sum(is.na(match(de.calls, de.genes))) / length(de.calls)
                }
                efdr
              }, pval, qval, de.genes)

  cbind(qval.cutoffs, y)
}

eFDRlist <- list()
pi0list <- list()

for (pDE in c(100, 1000, 2000)) {
  for (fc in c(1.5, 2, 4)) {
    cat("pDE=", pDE, "fc=", fc, "\n")
    sim.name <- sprintf("qvalXeFDR.p%d.pDE%d.n%d.fc%.1f.sdoff%.1f", p, pDE, n, fc, sdoffset) ## name identifying the previous parameters to store resulting files

    load(paste0(sim.name, ".RData"))

    qval.cutoffs <- seq(0, 0.1, by=0.001)
    all.eFDR <- list(tweeDEseq=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     DESeqPgOnly=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     DESeqPmax=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     DESeqCgOnly=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     DESeqCmax=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     edgeRdf20=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     edgeRqlfdf20=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     edgeRdf1=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                     edgeRqlfdf1=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims))
    all.pi0 <- list(tweeDEseq=rep(NA_real_, nsims),
                    DESeqPgOnly=rep(NA_real_, nsims),
                    DESeqPmax=rep(NA_real_, nsims),
                    DESeqCgOnly=rep(NA_real_, nsims),
                    DESeqCmax=rep(NA_real_, nsims),
                    edgeRdf20=rep(NA_real_, nsims),
                    edgeRqlfdf20=rep(NA_real_, nsims),
                    edgeRdf1=rep(NA_real_, nsims),
                    edgeRqlfdf1=rep(NA_real_, nsims))

    for (i in 1:nsims) {
      pval <- pv$tweeDEseq[[i]]
      qobj <- qvalue(pval)
      all.eFDR$tweeDEseq[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$tweeDEseq[i] <- qobj$pi0
      pval <- pv$DESeqPgOnly[[i]]
      qobj <- qvalue(pval)
      all.eFDR$DESeqPgOnly[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$DESeqPgOnly[i] <- qobj$pi0
      pval <- pv$DESeqPmax[[i]]
      qobj <- qvalue(pval)
      all.eFDR$DESeqPmax[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$DESeqPmax[i] <- qobj$pi0
      pval <- pv$DESeqCgOnly[[i]]
      qobj <- qvalue(pval)
      all.eFDR$DESeqCgOnly[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$DESeqCgOnly[i] <- qobj$pi0
      pval <- pv$DESeqCmax[[i]]
      qobj <- qvalue(pval)
      all.eFDR$DESeqCmax[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$DESeqCmax[i] <- qobj$pi0
      pval <- pv$edgeRdf20[[i]]
      qobj <- qvalue(pval)
      all.eFDR$edgeRdf20[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$edgeRdf20[i] <- qobj$pi0
      pval <- pv$edgeRqlfdf20[[i]]
      qobj <- qvalue(pval)
      all.eFDR$edgeRqlfdf20[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$edgeRqlfdf20[i] <- qobj$pi0
      pval <- pv$edgeRdf1[[i]]
      qobj <- qvalue(pval)
      all.eFDR$edgeRdf1[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$edgeRdf1[i] <- qobj$pi0
      pval <- pv$edgeRqlfdf1[[i]]
      qobj <- qvalue(pval)
      all.eFDR$edgeRqlfdf1[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
      all.pi0$edgeRqlfdf1[i] <- qobj$pi0
    }
  
    eFDRlist[[sim.name]] <- all.eFDR
    pi0list[[sim.name]] <- all.pi0
  }
}

save(eFDRlist, pi0list, file="DEsims.sdoff0.5.RData")
