library(RColorBrewer)

load("pickrell1simulationsResults.RData")

labelPlot <- function(lab, font, cex, offsetx=0.05, offsety=0.05) {
  par(xpd=TRUE)
  w <- par("usr")[2] - par("usr")[1]
  h <- par("usr")[4] - par("usr")[3]
  text(par("usr")[1]-w*offsetx, par("usr")[4]+h*offsety, lab, font=font, cex=cex)
  par(xpd=FALSE)
}

qqunif <- function(x, xlab="Expected Uniform Quantiles", ylab="Observed Uniform Quantiles",
                   col=2:(length(x)+1), lty=1, lwd=1, legend.args=NULL, ...) {
  n <- length(x[[1]])
  if (length(lty) == 1)
    lty <- rep(lty, length(x))
  if (length(lwd) == 1)
    lwd <- rep(lwd, length(x))
  plot((1:n) / n, sort(x[[1]]), xlab=xlab, ylab=ylab, type="l", col=col[1], lty=lty[1], lwd=lwd[1], ...)
  for (i in seq(along=x)[-1]) {
    n <- length(x[[i]])
    lines((1:n) / n, sort(x[[i]]), type="l", col=col[i], lty=lty[i], lwd=lwd[i])
  }

  abline(0, 1, lty=2)

  if (!is.null(legend.args))
    do.call("legend", legend.args)
}

pp <- colorRampPalette(brewer.pal(12, "Paired"))(12)
collist <- c(pp[6], pp[5], pp[7], pp[8], pp[12], pp[2], pp[1], pp[10], pp[9], pp[3])
names(collist) <- c("tweeDEseq", "edgeRdf20", "edgeRdf1", "edgeRqlfdf20", "edgeRqlfdf1",
                    "DESeqPgO", "DESeqPmax", "DESeqCgO", "DESeqCmax", "limma")

pdf(file="qqPickrell1sims.pdf", height=7, width=7)
par(mfrow=c(2,2), mar=c(4, 4, 2, 1))
qqunif(allGOFpvFvsFun, main="", lwd=3, col=collist,
legend.args=list("topleft", c("tweeDEseq", "edgeR - (default)", "edgeR - Df1"),
col=collist[c("tweeDEseq", "edgeRdf20", "edgeRdf1")], lwd=3, inset=0.02, cex=1))
labelPlot("a", 2, 2, 0.20, 0.03)
qqunif(allGOFpvFvsFcqn, main="", lwd=3, col=collist,
       legend.args=list("topleft", c("edgeR - QLF (default)", "edgeR - QLF Df1"),
       col=collist[c("edgeRqlfdf20", "edgeRqlfdf1")], lwd=3, inset=0.02, cex=1))
labelPlot("b", 2, 2, 0.20, 0.03)
qqunif(allGOFpvMvsMun, main="", lwd=3, col=collist,
       legend.args=list("topleft", c("DESeq - PgOn", "DESeq - Pmax"),
       col=collist[c("DESeqPgO", "DESeqPmax")], lwd=3, inset=0.02, cex=1))
labelPlot("c", 2, 2, 0.20, 0.03)
qqunif(allGOFpvMvsMcqn, main="", lwd=3, col=collist,
       legend.args=list("topleft", c("DESeq - CgOn", "DESeq - Cmax", "limma"),
       col=collist[c("DESeqCgO", "DESeqCmax", "limma")], lwd=3, inset=0.02, cex=1))
labelPlot("d", 2, 2, 0.20, 0.03)
dev.off()

pdf(file="histPickrell1sims.pdf", height=12, width=7)
metnames <- names(allPvaluesFvsFun)
metnames <- c("tweeDEseq", "edgeR\n(def)", "edgeR\nDf1", "edgeR-QLF\n(def)", "edgeR-QLF\nDf1",
              "DESeq\nPgOn", "DESeq\nPmax", "DESeq\nCgOn", "DESeq\nCmax", "limma")
set.seed(123)
par(mfrow=c(10,4), mar=c(2, 5, 2, 1))
for (i in 1:length(allPvaluesFvsFun)) {
  par(mar=c(2, 5, 2, 1))
  hist(allPvaluesFvsFun[[i]], axes=FALSE, main="", xlab="", ylab="", probability=TRUE)
  axis(1, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  axis(2, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  abline(h=1, col="red")
  mtext(metnames[[i]], side=2, line=2, font=2)
  if (i == 1) title("FvsF - UnNorm", cex.main=1.5)
  ## if (i==1) labelPlot("a", 2, 2, 0.2, 0.2)
  par(mar=c(2, 5, 2, 1))
  hist(allPvaluesFvsFcqn[[i]], axes=FALSE, main="", xlab="", ylab="", probability=TRUE)
  axis(1, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  axis(2, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  abline(h=1, col="red")
  if (i == 1) title("FvsF - CQN", cex.main=1.5)
  ## if (i==1) labelPlot("b", 2, 2, 0.2, 0.2)
  hist(allPvaluesMvsMun[[i]], axes=FALSE, main="", xlab="", ylab="", probability=TRUE)
  axis(1, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  axis(2, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  abline(h=1, col="red")
  if (i == 1) title("MvsM - UnNorm", cex.main=1.5)
  ## if (i==1) labelPlot("c", 2, 2, 0.2, 0.2)
  hist(allPvaluesMvsMcqn[[i]], axes=FALSE, main="", xlab="", ylab="", probability=TRUE)
  axis(1, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  axis(2, at=c(0, 0.5, 1), labels=c(0, 0.5, 1))
  abline(h=1, col="red")
  if (i == 1) title("MvsM - CQN", cex.main=1.5)
  ## if (i==1) labelPlot("d", 2, 2, 0.2, 0.2)
}
dev.off()
